-module(car2).
-include("bp.hrl").
-include("bp_ext.hrl").
-export([start/1]).
-import(bp, [is/1, match/1]).
-record(pos, {x, y}).

goal() -> {50, 50}. 

%% Movement sensors

head_north() ->
	{pos, _, Y} = bp:bSync(#rwb{wait=is(pos)}),
	{_, Y1} = goal(),
	if Y < Y1 ->
			bp:bSync(#rwb{request=[north]}),
			move_north();
	    true ->
			ok
	end,
	head_north().

head_south() ->
	{pos, _, Y} = bp:bSync(#rwb{wait=is(pos)}),
	{_, Y1} = goal(),
	if Y > Y1 ->
			bp:bSync(#rwb{request=[south]}),
			move_south();
	    true ->
			ok
	end,
	head_south().

head_west() ->
	{pos, X, _} = bp:bSync(#rwb{wait=is(pos)}),
	{X1, _} = goal(),
	if X < X1 ->
			bp:bSync(#rwb{request=[west]}),
			move_west();
	    true ->
			ok
	end,
	head_west().

head_east() ->
	{pos, X, _} = bp:bSync(#rwb{wait=is(pos)}),
	{X1, _} = goal(),
	if X > X1 ->
			bp:bSync(#rwb{request=[east]}),
			move_east();
	    true ->
			ok
	end,
	head_east().

%% Movement actuators

curr_pos(X, Y) ->
	receive
		{pos, Pid} when is_pid(Pid) ->
			Pid ! {X, Y},
			curr_pos(X, Y);
		{pos, X1, Y1} ->
			io:format(user, "@ ~p~n", [{X1, Y1}]),
			curr_pos(X1, Y1)
	end.

start_pos(X, Y) ->
	case whereis(pos) of
		undefined ->
			register(pos, spawn(fun() -> curr_pos(X, Y) end));
		Pid when is_pid(Pid) ->
			Pid ! {pos, X, Y}
	end.

get_pos() ->
	pos ! {pos, self()},
	receive E -> E end.

move_north() ->
	{X, Y} = get_pos(),
	Y1 = Y + 1,
	pos ! {pos, X, Y1}.

move_south() ->
	{X, Y} = get_pos(),
	Y1 = Y - 1,
	pos ! {pos, X, Y1}.

move_west() ->
	{X, Y} = get_pos(),
    X1 = X + 1,
	pos ! {pos, X1, Y}.

move_east() ->
	{X, Y} = get_pos(),
	X1 = X - 1,
	pos ! {pos, X1, Y}.

% Push a tick to the external queue every N milliseconds
ticker(N) ->
	receive 
		shutdown -> ok
	after N -> 
			bp_ext:push(tick),
			ticker(N)
	end.

on_tick() ->
	bp:bSync(#rwb{wait=[tick]}),
	{X, Y} = get_pos(),
	bp:bSync(#rwb{request=[{pos, X, Y}]}),
	on_tick().


%% periodical(S, N) ->
%% 	{pos, X, Y} = bp:bSync(#rwb{wait=is(pos)}),
%% 	io:format(user, "sup ! {pos, ~p, ~p, ~p}~n", [self(), X, Y]),
%% 	S ! {pos, self(), X, Y},
%% 	receive
%% 		E -> bp:bSync(#rwb{request=[E]})
%% 	after 1000 -> ok
%% 	end,
%% 	[bp:bSync(#rwb{wait=is(pos)}) || _ <- lists:seq(1,N)],
%% 	periodical(S, N).


report(S, P, N) ->
	{pos, X, Y} = bp:bSync(#rwb{wait=is(pos)}),
	io:format(user, "sup ! {pos, ~p, ~p, ~p}~n", [P, X, Y]),
	S ! {pos, P, X, Y},
	[bp:bSync(#rwb{wait=is(pos)}) || _ <- lists:seq(2,N)],
	report(S, P, N).

listen(N) ->
	bp:bSync(#rwb{wait=is(pos)}),
	io:format(user, "?~n", []),
	receive
		E ->
			io:format(user, "<- ~p~n", [E]),
			bp:bSync(#rwb{request=[E]})
	after 10 -> ok
	end,
	[bp:bSync(#rwb{wait=is(pos)}) || _ <- lists:seq(2,N)],
	listen(N).



on_warning()->
	{warn, Y1} = bp:bSync(#rwb{wait=is(warn)}),
	{pos, _, Y} = bp:bSync(#rwb{wait=is(pos)}),
	io:format(user, "dont cross ~p ~p~n", [Y1, Y]),
	if 
		Y >= Y1 ->
			hold();
		true ->
			on_warning()
	end.

hold() ->
	[bp:bSync(#rwb{wait=[tick], block=[north]}) || _ <- lists:seq(1,10)],
	on_warning().

echo() ->
	E = bp:bSync(#rwb{wait=?ALL}),
	io:format(user, "~p~n", [E]),
	echo().


start(Sup) ->
	start_pos(0, 0),
	bp:init(),
	bp_ext:start_idler(),
	bp_ext:start_queue(),
	bp:add(fun head_north/0, 1000),
	bp:add(fun head_south/0, 1001),
	bp:add(fun head_west/0,  1002),
	bp:add(fun head_east/0,  1003),
	bp:add(fun on_tick/0,    1004),
	P = bp:add(fun() -> listen(5) end, 1005),
	bp:add(fun() -> report(Sup, P, 10) end, 1006),
	bp:add(fun on_warning/0, 1010),
	bp:add(fun echo/0, 1011),
	bp:attach(spawn(fun() -> ticker(2000) end)),
	bp:start().
