-module(ctrl).
-include("bp.hrl").
-include("bp_ext.hrl").
-import(bp, [is/1, match/1]).
-export([start/0, test/1]).
-record(pos, {pid, x, y}).
-record(cross, {pid}).

monitor_line(Y1) ->
	{pos, P, _, Y} = bp:bSync(#rwb{wait=is(pos)}),
	if
		Y >= Y1 ->
			bp:bSync(#rwb{request=[{cross, P}]});
		true -> ok
	end,
	monitor_line(Y1).

monitor_crossing(N, Y1, L) ->
	{cross, P} = bp:bSync(#rwb{
        wait =
            fun({cross, E}) -> not(lists:member(E, L));
			   (_) -> false
			end}),
	if
		length(L) >= N -> 
			P ! {warn, Y1},
			io:format("~p ! warn ~p~n", [P, Y1]),
			monitor_crossing(N, Y1, L);
		true ->  monitor_crossing(N, Y1, [P | L])
	end.
	

listener() ->
	receive
		E when is_record(E, pos) -> 
			bp_ext:push(E),
			listener();
		shutdown -> 
			ok;
		_ -> 
			listener()
	end.

start() ->
	bp:init(),
	bp_ext:start_idler(),
	bp_ext:start_queue(),
	L = 15,
	bp:add(fun() -> monitor_line(L) end, 1000),
	bp:add(fun() -> monitor_crossing(1, L, []) end, 1001),
	Listen = bp:attach(spawn(fun listener/0)),
	bp:start(),
	Listen.

				   
test(Nodes) ->
	Sup = start(),
	[spawn(N, car2, start, [Sup]) || N <- Nodes].
