-module(landing_pad).

-behaviour(gen_server).

%% API
-export([start/1, left/0, right/0, pos/0, stop/0]).

%% gen_server callbacks
-export([init/1, handle_call/3, handle_cast/2, handle_info/2,
		 terminate/2, code_change/3]).

-define(SERVER, ?MODULE). 

-record(state, {x}).

%%%===================================================================
%%% API
%%%===================================================================

start(X) ->
	gen_server:start({local, ?SERVER}, ?MODULE, [X], []).

left()  -> gen_server:call(?SERVER, left),  rocket_panel:redraw().
right() -> gen_server:call(?SERVER, right), rocket_panel:redraw().
pos()   -> gen_server:call(?SERVER, pos).
stop()  -> gen_server:call(?SERVER, stop).	

%%%===================================================================
%%% gen_server callbacks
%%%===================================================================

init([X]) ->
	{ok, #state{x=X}}.

handle_call(left, _From, #state{x=X}) ->
	X1 = X-1,
	{reply, X1, #state{x=X1}};
handle_call(right, _From, #state{x=X}) ->
	X1 = X+1,
	{reply, X1, #state{x=X1}};
handle_call(pos, _From, State=#state{x=X}) ->
	{reply, X, State};
handle_call(stop, _From, State) ->
	{stop, normal, ok, State}.

handle_cast(_Msg, State) ->
	{noreply, State}.

handle_info(_Info, State) ->
	{noreply, State}.

terminate(_Reason, _State) ->
	ok.

code_change(_OldVsn, State, _Extra) ->
	{ok, State}.
