-module(rocket_panel).
-behavior(wx_object).

%% Client API
-export([start/0, start/1, redraw/0]).

%% wx_object callbacks
-export([init/1, terminate/2,  code_change/3,
		 handle_info/2, handle_call/3, handle_cast/2, 
		 handle_event/2, handle_sync_event/3]).

-include_lib("wx/include/wx.hrl").

-record(state, {panel, canvas}).

start(Parent) ->
    wx_object:start({local, ?MODULE}, ?MODULE, Parent, []).

redraw() ->
	wx_object:cast(?MODULE, draw).

init(Parent) ->
    wx:batch(fun() -> do_init(Parent) end).

do_init(Parent) ->
	Panel	  = wxPanel:new(Parent, []),
	MainSizer = wxBoxSizer:new(?wxVERTICAL),
	Canvas	  = wxPanel:new(Panel, [{style, ?wxFULL_REPAINT_ON_RESIZE}]),
	RowPanel  = wxPanel:new(Panel, []),
	RowSizer  = wxBoxSizer:new(?wxHORIZONTAL),
	LeftBtn	  = wxButton:new(RowPanel, ?wxID_BACKWARD, [{label, "<= left"}]),
	UpBtn	  = wxButton:new(RowPanel, ?wxID_UP, [{label, "up"}]),
	RightBtn  = wxButton:new(RowPanel, ?wxID_FORWARD, [{label, "right =>"}]),

	wxPanel:connect(Canvas, paint, [callback]),
    wxPanel:connect(RowPanel, command_button_clicked),

	wxSizer:add(RowSizer,  LeftBtn,  [{flag, ?wxALIGN_CENTER}]),
	wxSizer:add(RowSizer,  UpBtn,    [{flag, ?wxALIGN_CENTER}]),
	wxSizer:add(RowSizer,  RightBtn, [{flag, ?wxALIGN_CENTER}]),
	wxSizer:add(MainSizer, Canvas,   [{flag, ?wxEXPAND}, {proportion, 1}]),
	wxSizer:add(MainSizer, RowPanel, []),
	wxPanel:setSizer(RowPanel, RowSizer),
	wxPanel:setSizer(Panel, MainSizer),
	wxSizer:layout(MainSizer),
	{Panel, #state{panel=Panel, canvas=Canvas}}.

draw(Canvas) ->
	CDC = wxWindowDC:new(Canvas),
	wxDC:clear(CDC),
	wxDC:setBrush(CDC, ?wxRED_BRUSH),
	{X, Y} = rocket:pos(),
	X1 = landing_pad:pos(),
	wxDC:drawPolygon(CDC, [{10,0}, {16,6}, {16,16}, {20,20}, {0,20}, {4,16}, {4,6}, {10,0}], [{xoffset, X*20}, {yoffset, 200-Y*20}]),
	wxDC:setBrush(CDC, ?wxBLUE_BRUSH),
	wxDC:drawPolygon(CDC, [{0,0}, {20,0}, {16,4}, {16,16}, {20,20}, {0,20}, {4,16}, {4,4}, {0,0}], [{xoffset, X1*20}, {yoffset, 200}]),
	wxWindowDC:destroy(CDC).

handle_sync_event(#wx{event = #wxPaint{}}, _wxObj,
				  #state{canvas=Canvas}) ->
	draw(Canvas),
	ok.

handle_event(#wx{id=?wxID_BACKWARD}, State) ->
	bp_ext:push(user_left),
	{noreply, State};
handle_event(#wx{id=?wxID_FORWARD}, State) ->
	bp_ext:push(user_right),
	{noreply, State};
handle_event(#wx{id=?wxID_UP}, State) ->
	bp_ext:push(user_up),
	{noreply, State};
handle_event(E=#wx{}, State) ->
	io:format("Got event ~p~n", [E]),
	{noreply, State}.

handle_call(_, _, State=#state{canvas=C}) ->
	wx:batch(fun () -> draw(C) end),
	{noreply, State}.

handle_cast(_, State=#state{canvas=C}) ->
	wx:batch(fun () -> draw(C) end),
	{noreply, State}.

handle_info(M, State) ->
	io:format("Got info ~p~n", [M]),
	{noreply, State}.

code_change(_, _, State) ->
    {stop, ignore, State}.

terminate(_Reason, _State) ->
	io:format("Deleting ~p~n", [?MODULE]),
	bp:shutdown(),
	wx:destroy(),
	rocket:stop(),
	landing_pad:stop(),
    ok.

start() ->
	WX = wx:new(),
	Frame = wxFrame:new(WX, -1, "Land the rocket!", [{size, {250, 250}}]),
	start(Frame),
	wxFrame:show(Frame),
	Frame.
